/***************************************************************************
 *   Copyright (C) 2004 by Stephen Anthony                                 *
 *   stephena@users.sourceforge.net                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

using namespace std;

#include <iostream>
#include <fstream>

#include <klocale.h>
#include <kprocess.h>

#include <qtextstream.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qprogressdialog.h>

#include "kstella.h"
#include "game.h"
#include "gamelist.h"
#include "gamelistdoc.h"
#include "md5.h"


GameListDoc::GameListDoc( QWidget* parent, const char* name )
			:QObject( parent, name )
{
	list = new GameList();
	if( !list )
	{
		cerr << "Couldn't get memory for gameList in GameListDoc::GameListDoc\n";
		return;
	}

	// game statistics info
	numTotalGames = 0;
	numActualGames = 0;

	config = kapp->config();
}


GameListDoc::~GameListDoc( void )
{
	if( list )
		delete list;
}


void GameListDoc::load( void )
{
	config->setGroup( "Emulator Defaults" );
	QString romDir = config->readEntry( "RomDir" );
	QString cache  = QDir::homeDirPath() + "/.stella/kstella.cache";

	// check to see if the cache file is invalid
	// this happens when the rom directory has been changed since
	// kstella was last run
	config->setGroup( "General Options" );
	QDateTime lastModificationTime = config->readDateTimeEntry( "Modification Time", 0 );
	QDateTime modificationTime     = QFileInfo(romDir).lastModified();

	bool cacheStale = modificationTime > lastModificationTime;

	if( QFile::exists( cache ) )
	{
		// only load from cache if it hasn't changed
		if( cacheStale )
		{
			QMessageBox::information( 0, "Rom directory modified.",
				i18n("KStella detected a change in your rom directory.\nThe game list will be updated.") );
			loadFromDisk();
		}
		else
			loadFromCache();
	}
	else
	{
		QMessageBox::information( 0, "Build game list",
			i18n("Building game list (this could take some time).") );
		loadFromDisk();
	}
}


void GameListDoc::reload( void )
{
	loadFromDisk();
}


GameList* GameListDoc::gameList( void )
{
	return list;
}


void GameListDoc::loadFromCache( void )
{
	QString cache = QDir::homeDirPath() + "/.stella/kstella.cache";
	QFile f( cache );
	if( f.open( IO_ReadOnly ) == 0 )
	{
		cerr << "Couldn't find cache\n";
		return;
	}

	numTotalGames = numActualGames = 0;
	list->clear();

	QTextStream in( &f );
	int limit = in.readLine().toInt();

	for( int i = 0; i < limit; ++i )
	{
		Game* g = new Game();
		bool avail = (in.readLine().toInt() == 1);
		g->setAvailable( avail );
		g->setRom( in.readLine() );
		g->setMd5( in.readLine() );
		g->setName( in.readLine() );
		g->setRarity( in.readLine() );
		g->setManufacturer( in.readLine() );
		g->setNote( in.readLine() );

		list->insert( g );
		numTotalGames++;
		if( avail )
			numActualGames++;
	}
	f.close();
}


void GameListDoc::loadFromDisk( void )
{
	numTotalGames = numActualGames = 0;
	list->clear();

	// get the file location of the Stella data file 'stella.pro'
	// it can be either located in the users home directory as '.stella/stella.pro',
	// or in the global location '/etc/stella.pro'
	QString properties, homeStellaPro, systemStellaPro;

	homeStellaPro    = QDir::homeDirPath() + "/.stella/stella.pro";
	systemStellaPro  = "/etc/stella.pro";

	if( QFile::exists( homeStellaPro ) )
	{
		properties = homeStellaPro;
	}
	else if( QFile::exists( systemStellaPro ) )
	{
		properties = systemStellaPro;
	}
	else
	{
		QMessageBox::information( 0, "Missing File",
			i18n("No stella.pro file available.\nPlease see the documentation.") );
		return;
	}

	// get the directory where the ROMs are located
	config->setGroup( "Emulator Defaults" );
	loadGameProperties( properties );
	updateGameList( config->readEntry( "RomDir" ) );

	createCache();
}


void GameListDoc::loadGameProperties( const QString& filename )
{
	// data pair of variable and its value
	QString variable, value;
	// temporary string to hold the data read from the file
	QString s;
	// end of variable and start of value indexes
	int two, three;

	// Make sure to recount the number of games
	numTotalGames = 0;

	// Try to detect errors in the placement of "" symbol
	bool goodData = false;

	Game* game = new Game();
	QFile f( filename );
	if( f.open( IO_ReadOnly ) != 0 )    // file opened successfully
	{
		QTextStream t( &f );        // use a text stream

		while( !t.eof() )           // until end of file...
		{
			s = t.readLine();       // line of text excluding '\n'
			s = s.stripWhiteSpace();

			// ignore null lines and lines beginning with semicolons
			if( (s.length() == 0) || s.startsWith( ";" ) )
				continue;

			// "" line indicates end of record
			if( s == "\"\"" && goodData )
			{
				list->insert( game );
				game = new Game();
				numTotalGames++;
				goodData = false;
				continue;
			}

			// good data - read the variable and value
			two = s.find( "\"" , 1 );
			three = s.find( "\"", two + 1 );
			variable = s.mid( 1, two - 1 );
			value = s.mid( three + 1, s.length() - three - 2 );
			goodData = true;

			if( variable == "Cartridge.MD5" )
				game->setMd5( value );
			else if( variable == "Cartridge.Name" )
				game->setName( value );
			else if( variable == "Cartridge.Rarity" )
				game->setRarity( value );
			else if( variable == "Cartridge.Manufacturer" )
				game->setManufacturer( value );
			else if( variable == "Cartridge.Note" )
				game->setNote( value );
		}
		f.close();
	}
}


void GameListDoc::updateGameList( const QString& romDir )
{
	QDir d( romDir, QString::null, QDir::Unsorted, QDir::Files );
	const QFileInfoList* infolist = d.entryInfoList();
	QFileInfoListIterator it( *infolist );
	QFileInfo* fi;

	while( (fi = it.current()) != 0 )
	{
		bool gameFound = false;

		QString filename = fi->filePath();
		int size = fi->size();
		ifstream in(filename, ios_base::binary);
		unsigned char* image = new unsigned char[size];
		in.read((char*)image, size);
		in.close();
		QString md5 = MD5( image, size );
		delete[] image;
		
		for( int i = 0; i < list->size(); ++i )
		{
			Game* g = list->at(i);
			if( md5 == g->md5() )
			{
				if( g->isAvailable() )
					cerr << "Duplicate found: " << g->rom() << ", " << filename << endl;

				g->setRom( filename );
				g->setAvailable( true );
				numActualGames++;
				gameFound = true;
				break;
			}
		}

		// a game was found that didn't have a stella.pro entry
		// we still want it in the gamelist
		if( !gameFound )
		{
			Game* g = new Game();
			g->setRom( filename );
			g->setMd5( md5 );
			g->setName( fi->fileName() );
			g->setAvailable( true );
			numTotalGames++;
			numActualGames++;
			list->insert( g );
		}
			
		++it;
	}
}


QString GameListDoc::gameStatistics( void )
{
	QString gameStats;

	gameStats =  "Stella Properties File by Erik Kovach\n";
	gameStats += "http://home.earthlink.net/~voch/atari.html\n";
	gameStats += "\n";
	gameStats += "Total Games:  ";
	gameStats += QString::number( numTotalGames );
	gameStats += "\n";
	gameStats += "Available Games:  ";
	gameStats += QString::number( numActualGames );
	gameStats += "\n";
	gameStats += "Unavailable Games:  ";
	gameStats += QString::number( numTotalGames - numActualGames );
	gameStats += "\n";

	return gameStats;
}


void GameListDoc::createCache( void )
{
	QString path  = QDir::homeDirPath() + "/.stella";
	QString cache = path + "/kstella.cache";

	// create a .stella directory if one doesn't exist
	if( !QFile::exists( path ) )
	{
		QDir dir;
		if( !dir.mkdir( path ) )
		{
			cerr << "Couldn't create .stella directory.\n";
			return;
		}
	}

	QFile f( cache );
	if( f.open( IO_WriteOnly ) == 0 )
	{
		cerr << "Couldn't create cache file.\n";
		return;
	}

	QTextStream out( &f );
	out << list->size() << endl;
	for( int i = 0; i < list->size(); ++i )
	{
		Game* g = list->at(i);
		if( g->isAvailable() )
			out << 1 << endl;
		else
			out << 0 << endl;
		out <<	g->rom() << endl <<
				g->md5() << endl <<
				g->name() << endl <<
				g->rarity() << endl <<
				g->manufacturer() << endl <<
				g->note() << endl;
	}
	f.close();

	// now indicate that the cache has been updated
	config->setGroup( "Emulator Defaults" );
	QString romDir = config->readEntry( "RomDir" );
	QDateTime modificationTime = QFileInfo(romDir).lastModified();
	config->setGroup( "General Options" );
	config->writeEntry( "Modification Time", modificationTime );
}
