/***************************************************************************
 *   Copyright (C) 2004 by Stephen Anthony                                 *
 *   stephena@users.sourceforge.net                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

using namespace std;

#include <iostream>

#include <kconfig.h>
#include <kapplication.h>
#include <klocale.h>

#include <qlistview.h>
#include <qheader.h>
#include <qtimer.h>

#include "kstella.h"
#include "game.h"
#include "gamelist.h"
#include "qindexeditem.h"
#include "gamelistview.h"


GameListView::GameListView( QWidget* parent, const char* name )
			: QListView( parent, name )
{
	// set up the config for this application
	config = kapp->config();

	config->setGroup( "General Options" );
	int nameColWidth         = config->readNumEntry( "NameColWidth", 80 );
	int rarityColWidth       = config->readNumEntry( "RarityColWidth", 80 );
	int manufacturerColWidth = config->readNumEntry( "ManufacturerColWidth", 80 );

	// set the list to highlight the entire row, not just a column
	setAllColumnsShowFocus(true);
	// add the 4 columns
	addColumn( i18n("Name"), nameColWidth );
	addColumn( i18n("Rarity"), rarityColWidth );
	addColumn( i18n("Manufacturer"), manufacturerColWidth );

	// if the highlighted item changes, update the tab item(s)
	connect( this, SIGNAL(selectionChanged(QListViewItem *)),
			 this, SLOT(slotSelectionChanged(QListViewItem *)) );
	// if the highlighted item is selected, run the emulator for that game
	connect( this, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(slotEmulate(QListViewItem *)) );
    connect( this, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(slotEmulate(QListViewItem *)) );

	// set up timer so that we don't get too many signals that end up eating all the CPU
	selectionTimer = new QTimer( this );
	connect( selectionTimer, SIGNAL(timeout()), this, SLOT(slotSelectionTimerFinished()) );
}


GameListView::~GameListView( void )
{
	// save the column positions
	QHeader* h = header();
	int nameColWidth         = h->sectionSize( 0 );
	int rarityColWidth       = h->sectionSize( 1 );
	int manufacturerColWidth = h->sectionSize( 2 );

	config->setGroup( "General Options" );
	config->writeEntry( "NameColWidth", nameColWidth );
	config->writeEntry( "RarityColWidth", rarityColWidth );
	config->writeEntry( "ManufacturerColWidth", manufacturerColWidth );

	if( selectionTimer )
		delete selectionTimer;
}


void GameListView::showRarity( bool show )
{
	int rarityColWidth = 0;

	if( header()->sectionSize( 1 ) > 0 )
		rarityColWidth = header()->sectionSize( 1 );

	if( show )
		resizeColumn( 1, (rarityColWidth > 0 ? rarityColWidth : 80) );
	else
		resizeColumn( 1, 0 );
}


void GameListView::showManufacturer( bool show )
{
	int manufacturerColWidth = 0;

	if( header()->sectionSize( 2 ) > 0 )
		manufacturerColWidth = header()->sectionSize( 2 );

	if( show )
		resizeColumn( 2, (manufacturerColWidth > 0 ? manufacturerColWidth : 80) );
	else
		resizeColumn( 2, 0 );
}


void GameListView::showNote( bool show )
{
	int noteColWidth = 0;

	if( header()->sectionSize( 3 ) > 0 )
		noteColWidth = header()->sectionSize( 3 );

	if( show )
		resizeColumn( 3, (noteColWidth > 0 ? noteColWidth : 80) );
	else
		resizeColumn( 3, 0 );
}


void GameListView::resizeColumn( int column, int size )
{
	QHeader* h = header();
	h->resizeSection( column, size );

	// make sure that 0 width columns can't be resized
	h->setResizeEnabled( (size == 0  ? false: true), column );
}


void GameListView::setGameList( GameList* list )
{
	gameList = list;
}


void GameListView::update( void )
{
	if( !gameList )
		return;

	// remove all items from the view
	clear();

	// does the user want to see ALL, AVAILABLE or UNAVAILABLE roms
	int romView = KStella::romView();

	// put the game into the viewlist if:
	//    1. ALL roms are desired
	// or 2. AVAILABLE roms are desired and the rom is AVAILABLE
	// or 3. UNAVAILABLE roms are desired and the rom is UNAVAILABLE
	for( int i = 0; i < gameList->size(); ++i )
	{
		Game* g = gameList->at( i );
		if( g == (Game*) NULL )
			return;

		if( (romView == KStella::ALL) ||
			((romView == KStella::AVAILABLE) && (g->isAvailable())) ||
			((romView == KStella::UNAVAILABLE) && (!g->isAvailable())))
			(void) new QIndexedItem( this, i, g->name(), g->rarity(), g->manufacturer() );
	}

	if( firstChild() != 0 )
		setSelected( firstChild(), true );

	// now give the listview the current focus
	setFocus();
}


Game* GameListView::currentGame( void )
{
	if( !gameList )
		return (Game*) NULL;

	// get the list index of the current game
	QIndexedItem* item = (QIndexedItem*) currentItem;
	if( item == 0 )
		return (Game*) NULL;

	return gameList->at( item->getIndex() );
}


void GameListView::slotSelectionChanged( QListViewItem* item )
{
	// set the current item so we don't have to search for it later
	// makes lookup of items slightly faster
	currentItem = item;

	// wait a while before registering this as a selection so that we don't hog the CPU
    selectionTimer->start( 50, TRUE );
}


void GameListView::slotSelectionTimerFinished( void )
{
	Game* g = currentGame();
	if( g == (Game*) NULL )
		return;

	config->setGroup( "General Options" );
	bool gameinfo = config->readBoolEntry( "Game Info", true );
	if( gameinfo )
	{
		cerr << "\nName   : " << (const char *) g->name() <<
				"\nRom    : " << (const char *) g->rom()  <<
				"\nmd5sum : " << (const char *) g->md5()  << endl;
	}

	// emit a signal indicating that a new game has been selected
	emit( changeSelection( g ) );
}


void GameListView::slotEmulate( QListViewItem* )
{
	emit( startEmulation() );
}


void GameListView::keyPressEvent( QKeyEvent* e )
{
	switch( e->key() )
	{
		case Key_Left:
			emit changeTab( 0 );
			break;

		case Key_Right:
			emit changeTab( 1 );
			break;

		default:
			QListView::keyPressEvent(e);
			break;
	}
}
