/***************************************************************************
 *   Copyright (C) 2004 by Stephen Anthony                                 *
 *   stephena@users.sourceforge.net                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

using namespace std;

#include <iostream>
#include <kapplication.h>
#include <knuminput.h>

#include <qstring.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qslider.h>
#include <qcombobox.h>
#include <qlabel.h>

#include "settings.h"


Settings::Settings( QWidget* parent, const char* name, const QString& title, const QString& group )
		: SettingsDlg( parent, name, TRUE )
{
	glAspectSelector->setPrecision( 1 );
	glAspectSelector->setMinValue( 0.1 );
	glAspectSelector->setMaxValue( 10.0 );

	// set the current group to use - this allows individual game settings
	_group = group;

	setCaption( title );
	readConfig( _group );
}


Settings::~Settings( void )
{
}


void Settings::slotDefaultsClicked( void )
{
	// updating the defaults -- use hard coded values
	if( _group == "Game Defaults" )
	{
		// these are the "default" settings
		videoDriverComboBox->setCurrentItem( 0 );  // software
		slotVideoDriverActivated( 0 );
		glFilterComboBox->setCurrentItem( 0 );     // nearest
		glAspectSelector->setValue( 2.0f );
		paletteComboBox->setCurrentItem( 0 );      // standard palette
		fpsSelector->setValue( 60 );
		zoomWinSelector->setValue( 2 );
		fullScreenButton->setChecked( FALSE );
		glFSMaxButton->setChecked( TRUE );

		volumeSelector->setValue( 50 );
		fragComboBox->setCurrentItem( 2 );         // 512 bytes
		volumeDisableButton->setChecked( FALSE );
		soundDisableButton->setChecked( FALSE );

		accurateTimingButton->setChecked( FALSE );
		grabMouseButton->setChecked( TRUE );
		hideCursorButton->setChecked( TRUE );

		paddleComboBox->setCurrentItem( 0 );       // Paddle 0
	}
	else
		readConfig( "Game Defaults" );
}


void Settings::slotOKClicked( void )
{
	// write the configuration immediately, not upon program exit
	writeConfig( _group );
	accept();
}


void Settings::slotVolumeDisableToggled( bool toggle )
{
	// (De)activate the volume selector
	volumeLabel->setEnabled( !toggle );
	volumeSelector->setEnabled( !toggle );
}

void Settings::slotSoundDisableToggled( bool toggle )
{
	// (De)activate everything related to sound
	// If we are reactivating, make sure to respect the volume disable setting
	if( !toggle )
	{
		if( !volumeDisableButton->isChecked() )
		{
			volumeLabel->setEnabled( !toggle );
			volumeSelector->setEnabled( !toggle );
		}
		fragLabel->setEnabled( !toggle );
		fragComboBox->setEnabled( !toggle );
		volumeDisableButton->setEnabled( !toggle );
	}
	else
	{
		volumeLabel->setEnabled( !toggle );
		volumeSelector->setEnabled( !toggle );
		fragLabel->setEnabled( !toggle );
		fragComboBox->setEnabled( !toggle );
		volumeDisableButton->setEnabled( !toggle );
	}
}


void Settings::slotVideoDriverActivated( int index )
{
	// Deactivate filtering and aspect in software mode
	bool toggle = (index == 0) ? false : true;
	glFilterLabel->setEnabled( toggle );
	glFilterComboBox->setEnabled( toggle );
	glAspectLabel->setEnabled( toggle );
	glAspectSelector->setEnabled( toggle );
	glFSMaxButton->setEnabled( toggle );
}


void Settings::readConfig( const QString& group )
{
	// temporary variables
	QString string;
	int i;
	bool b;
	double d;

	config = kapp->config();
	config->setGroup( group );

	// test for a null case
	string = config->readEntry( "FPS" );
	if( string.isNull() )
		config->setGroup( "Game Defaults" );

	///////////////////
	// Video settings
	i = config->readNumEntry( "VideoDriver" );
	videoDriverComboBox->setCurrentItem( i );
	slotVideoDriverActivated( i );
	i = config->readNumEntry( "VideoFilter" );
	glFilterComboBox->setCurrentItem( i );
	d = config->readDoubleNumEntry( "VideoAspect" );
	glAspectSelector->setValue( d );
	i = config->readNumEntry( "Palette" );
	paletteComboBox->setCurrentItem( i );
	i = config->readNumEntry( "FPS" );
	fpsSelector->setValue( i );
	i = config->readNumEntry( "Zoom" );
	zoomWinSelector->setValue( i );
	b = config->readBoolEntry( "Fullscreen" );
	fullScreenButton->setChecked( b );
	b = config->readBoolEntry( "FSMax" );
	glFSMaxButton->setChecked( b );

	///////////////////
	// Sound settings
	i = config->readNumEntry( "Volume" );
	volumeSelector->setValue( i );
	i = config->readNumEntry( "Fragsize" );
	fragComboBox->setCurrentItem( i );
	b = config->readBoolEntry( "VolumeDisable" );
	volumeDisableButton->setChecked( b );
	b = config->readBoolEntry( "SoundDisable" );
	soundDisableButton->setChecked( b );

	///////////////////
	// Misc settings
	b = config->readBoolEntry( "AccurateTiming" );
	accurateTimingButton->setChecked( b );
	b = config->readBoolEntry( "Grabmouse" );
	grabMouseButton->setChecked( b );
	b = config->readBoolEntry( "Hidecursor" );
	hideCursorButton->setChecked( b );

	///////////////////
	// Paddle settings
	i = config->readNumEntry( "MouseIsPaddle" );
	paddleComboBox->setCurrentItem( i );
}


void Settings::writeConfig( const QString& group )
{
	QString args, s;

	config->setGroup( group );

	///////////////////
	// Video settings
	config->writeEntry( "VideoDriver", videoDriverComboBox->currentItem() );
	config->writeEntry( "VideoFilter", glFilterComboBox->currentItem() );
	config->writeEntry( "VideoAspect", glAspectSelector->value() );
	config->writeEntry( "Palette", paletteComboBox->currentItem() );
	config->writeEntry( "FPS", fpsSelector->value() );
	config->writeEntry( "Zoom", zoomWinSelector->value() );
	config->writeEntry( "Fullscreen", fullScreenButton->isChecked() );
	config->writeEntry( "FSMax", glFSMaxButton->isChecked() );

	///////////////////
	// Sound settings
	config->writeEntry( "Volume", volumeSelector->value() );
	config->writeEntry( "Fragsize", fragComboBox->currentItem());
	config->writeEntry( "VolumeDisable", volumeDisableButton->isChecked() );
	config->writeEntry( "SoundDisable", soundDisableButton->isChecked() );

	///////////////////
	// Misc settings
	config->writeEntry( "AccurateTiming", accurateTimingButton->isChecked() );
	config->writeEntry( "Grabmouse", grabMouseButton->isChecked() );
	config->writeEntry( "Hidecursor", hideCursorButton->isChecked() );

	///////////////////
	// Paddle settings
	config->writeEntry( "MouseIsPaddle", paddleComboBox->currentItem() );

	//////////////////////////////////////
	// Now write the commandline arguments
	//////////////////////////////////////

	// "video" commandline argument
	s = videoDriverComboBox->currentText();
	if( s == "Software" )
		args += " -video soft";
	else if( s == "OpenGL" )
		args += " -video gl";

	// "gl_filter" commandline argument
	s = glFilterComboBox->currentText();
	if( s == "Nearest" )
		args += " -gl_filter nearest";
	else if( s == "Linear" )
		args += " -gl_filter linear";

	// "gl_aspect" command line argument
	s.setNum( glAspectSelector->value() );
	args += " -gl_aspect " + s;

	// "palette" commandline argument
	s = paletteComboBox->currentText();
	if( s == "Standard" )
		args += " -palette standard";
	else if( s == "Original" )
		args += " -palette original";
	else if( s == "Z26" )
		args += " -palette z26";

	// "fps" command line argument
	s.setNum( fpsSelector->value() );
	args += " -framerate " + s;

	// "zoom" command line argument
	s.setNum( zoomWinSelector->value() );
	args += " -zoom " + s;

	// "fullscreen" command line argument
	args += " -fullscreen ";
	args += fullScreenButton->isChecked() ? "true" : "false";

	// "gl_fsmax" command line argument
	args += " -gl_fsmax ";
	args += glFSMaxButton->isChecked() ? "true" : "false";

	// "sound" commandline argument
	if( !soundDisableButton->isChecked() )
		args += " -sound true ";
	else
		args += " -sound false ";

	// "volume" command line argument
	if( !volumeDisableButton->isChecked() )
	{
		s.setNum( volumeSelector->value() );
		args += " -volume " + s;
	}
	else
		args += " -volume -1 ";

	// "fragsize" commandline argument
	args += " -fragsize ";
	args += fragComboBox->currentText();

	// "accurate" command line argument
	args += " -accurate ";
	args += accurateTimingButton->isChecked() ? "true" : "false";

	// "grabmouse" command line argument
	args += " -grabmouse ";
	args += grabMouseButton->isChecked() ? "true" : "false";

	// "hidecursor" command line argument
	args += " -hidecursor ";
	args += hideCursorButton->isChecked() ? "true" : "false";

	// "paddle" command line argument
	s.setNum( paddleComboBox->currentItem() );
	args += " -paddle " + s;

	// write all the game command line arguments into one big command
	config->writeEntry( "GameArgs", args );

	// make sure to actually write this config NOW not upon program exit
	config->sync();
}
